<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services\Retorno;

use Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;
use Geral\Entities\Empresa;

class MultiPagBradesco extends LeitorCnab {
    CONST ID_HEADER_ARQUIVO = '0';
    CONST ID_HEADER_LOTE = '1';
    CONST ID_REGISTRO_DETALHE = '3';
    CONST INSCRICAO_FISICA = '1';

    CONST DARF_NORMAL = '16';
    CONST GPS = '17';
    CONST DARF_SIMPLES = '18';
    CONST PREVIDENCIA_SOCIAL = '22';
    CONST CODIGO_COMPENSACAO_SANTANDER = '237';

    private $cdagencia;
    private $cdconta;
    private $ocorrenciaHeader;
    private $contHeaderLote = 0;
    private $nrlanbolpag = 0;
    private $cdfilial = 0;
    private $boletoValido;
    /**
     * @param $arquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function __construct($arquivo, $cdempresa, $cdbanco){
        $this->cdbanco = $cdbanco;
        $this->arquivo = $arquivo;
        $this->cdempresa = $cdempresa;
    }

    private function validaArquivo($linha, $estruturaArquivo){

        $valido  = true;
        $idReg = substr($linha,7,1);
        if($this->contlinha == 1 and $idReg != self::ID_HEADER_ARQUIVO ){
            self::gravaErros($this->contlinha, "Erro de identificação do registro header.",true);
            $valido = false;
        }
        if($this->contlinha > 1 and (! in_array($idReg, $estruturaArquivo))){
            self::gravaErros($this->contlinha,"Identificação do registro não é válida",true);
            $valido = false;
        }

        return $valido;
    }

    public function getEstruturaArquivo(){
        return array(
                self::ID_HEADER_ARQUIVO,
                self::ID_HEADER_LOTE,
                self::ID_REGISTRO_DETALHE
            );
    }

    public function leArquivo($nrorg)
    {
        try {
            foreach ($this->arquivo as $linha) {
                $this->contlinha++;
                if(empty($linha)){
                    self::gravaErros($this->contlinha, "Linha em Branco.",true);
                }else{
                    $this->retorno = new Retorno();

                    // Busca o Tipo de Registro do Arquivo
                    $idRegistro = substr($linha,7,1);
                    if(self::validaArquivo($linha, $this->getEstruturaArquivo())){
                        switch ($idRegistro) {
                            case(self::ID_HEADER_ARQUIVO):
                                self::leHeaderArquivo($linha, $nrorg);
                                break;

                            case(self::ID_HEADER_LOTE):
                                self::leHeaderLote($linha);
                                break;

                            case(self::ID_REGISTRO_DETALHE):
                                self::leRegistroDetalhe($linha, $nrorg);
                                break;
                        }
                    }
                    $this->retornos[] = $this->retorno;
                }
            }
            self::preparaOcorrencias($this->cdbanco, $nrorg);

        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param
     * Retona 3 caso o registro não tenha nenhuma inconsistência,
     * e 2 caso possua alguma inconsistência.(Ver Layout p/ mais informações)
     * @return int
     */
    private function nivel($codigo){

        $conterro = 0;

        //Percorre as informações do retorno de duas em duas posições
        while($conterro < 10 ){

            $cderro = trim(substr($codigo,$conterro,2));
            //Verifica se a informação não está vazia
            if(! empty($cderro)){
                //Verifica se a informação está inconsistente
                if(! in_array($cderro,array('BD','BT','BU','BV','BW','FS',
                                            'GS','GT','GU','GV','GX','JB','HA'))){
                    return 2; //INCONSISTÊNCIA INVALIDA O REGISTRO
                }
            }
            $conterro+=2;
        }
        return 3; //CONSISTÊNCIA A TAREFA FOI EXECUTADA
    }

    private function getIncricaoEmpresa($cdempresa, $nrorg){
        $empresa = Empresa::findOneByCdempresaAndNrorg($cdempresa, $nrorg);
        if($empresa)
           return $empresa->getNrinsjurempr();
        else
            throw new \Exception ('Empresa não Encontrada.');
    }

    private function leHeaderArquivo($header, $nrorg){
        if(substr($header,0,3) != self::CODIGO_COMPENSACAO_SANTANDER)
            self::gravaErros($this->contlinha,'Banco não Cadastrado.', true);
        $nrinsjurempr = self::getIncricaoEmpresa($this->cdempresa, $nrorg);
        $tipoInscricao = substr($header,17,1);
        if(empty($tipoInscricao))
            self::gravaErros($this->contlinha,'Tipo de Inscrição da Empresa não Informado.', true);

        if ($tipoInscricao == self::INSCRICAO_FISICA){
            $codigoInscricao = substr($header,21,14);
        }else{
            $codigoInscricao = substr($header,18,14);
        }

        if ($nrinsjurempr <> $codigoInscricao){
            self::gravaErros($this->contlinha, 'Inscrição informada no retorno diverge da inscrição cadastrada no sistema.',true);
        }
        $tipoInscricao = substr($header,142,1);
        if ($tipoInscricao <> '2'){
            self::gravaErros($this->contlinha, 'Tipo de retorno inválido.',true);
        }
    }

    private function leHeaderLote($header){

        $this->cdagencia = substr($header,52,5);
        $this->cdconta = substr($header,58, 12);
        $this->ocorrenciaHeader = substr($header, 230, 10);
    }

    private function leRegistroDetalhe($registro, $nrorg){
        $this->boletoValido = false;
        $segmento = substr($registro, 13, 1);
        switch($segmento){
            case 'A': $this->segmentoA($registro);
                break;
            case 'B': $this->segmentoB($registro);
                break;
            case 'J': $this->segmentoJ($registro);
                break;
            case 'O': $this->segmentoO($registro);
                break;
            case 'N': $this->segmentoN($registro);
                break;
        }

        $idtitulopag = substr($registro,414,9);

        if ($this->boletoValido){
            $this->retorno->setCdagencia($this->cdagencia);
            $this->retorno->setCdconta($this->cdconta);
            $this->retorno->setBoleto('BOLETO');
            $this->retorno->setNrlanbolpag($this->nrlanbolpag);
            $this->retorno->setCdfilial($this->cdfilial);
            $this->retorno->setCdlinha(str_pad($this->contlinha,'0',STR_PAD_LEFT));
        }
    }

    private function segmentoA($registro){
        $this->boletoValido = true;
        $this->cdfilial = substr($registro, 73, 4);
        $this->nrlanbolpag = substr($registro, 77, 10);
        $nossoNumero = substr($registro, 134, 20);
        $this->retorno->setNossonumero($nossoNumero);
        $this->retorno->setDtpagamento(substr($registro, 154, 8));
        $vrpago = floatval(substr($registro, 119, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        if (empty($nossoNumero)){
            self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
            parent::setStatusArquivo(false);
        }
    }

    private function segmentoB($registro){
        $registroAnterior = count($this->retornos) - 1;
        $this->cdfilial = $this->retornos[$registroAnterior]->getCdfilial();
        $this->nrlanbolpag = $this->retornos[$registroAnterior]->getNrlanbolpag();
        $this->retorno->setNossonumero(substr($registro, 215, 15));
        $this->retorno->setDtvencimento(substr($registro, 127, 8));
        $vrabatimento = floatval(substr($registro, 150, 13));
        $vrdesconto = floatval(substr($registro, 165, 13));
        $this->retorno->setVrdesconto(round($vrdesconto / 100, 2));
        $vracrescimo = floatval(substr($registro, 180, 13));
        $vrmulta = floatval(substr($registro, 195, 13));
        $this->retorno->setVracrescimo($vracrescimo);
    }

    private function segmentoJ($registro){
        $this->boletoValido = true;
        $this->cdfilial = substr($registro, 182, 4);
        $this->nrlanbolpag = substr($registro, 186, 10);
        $this->retorno->setDtvencimento(substr($registro, 91, 8));
        $this->retorno->setNossonumero(substr($registro, 203, 20));
        $vrdesconto = floatval(substr($registro, 114, 15));
        $this->retorno->setVrdesconto(round($vrdesconto / 100, 2));
        $vracrescimo = floatval(substr($registro, 129, 15));
        $this->retorno->setVracrescimo($vracrescimo);
        $this->retorno->setDtpagamento(substr($registro, 144, 8));
        $vrpago = floatval(substr($registro, 152, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        $this->retorno->setCdconta(substr($registro, 29, 12));
        $digitoVerioficadorAgencia = substr($registro, 42, 1);
        $nrRetorno = trim(substr($registro, 202, 20));
        if (empty($nrRetorno)){
            self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
            parent::setStatusArquivo(false);
        }
    }

    private function segmentoN($registro){
        $this->retorno->setDtpagamento(substr($registro, 87, 8));
        $vrpago = floatval(substr($registro, 95, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        $tipoTributo = substr($registro, 132, 2);
        if ($tipoTributo == self::DARF_NORMAL){
            $vrmulta = floatval(substr($registro, 174, 13));
            $juros = floatval(substr($registro, 189, 13));
            $vracrescimo = $vrmulta + $juros;
        }elseif ($tipoTributo == self::GPS){
            $vracrescimo = floatval(substr($registro, 170, 13));
        }elseif ($tipoTributo == self::DARF_SIMPLES){
            $vrmulta = floatval(substr($registro, 179, 13));
            $juros = floatval(substr($registro, 194, 13));
            $vracrescimo = $vrmulta + $juros;
        }elseif ($tipoTributo == self::GARE){
            $vrmulta = floatval(substr($registro, 201, 13));
            $juros = floatval(substr($registro, 215, 13));
            $vracrescimo = $vrmulta + $juros;
            $this->retorno->setDtvencimento(substr($registro, 135, 8));
        }else{
            $vracrescimo = floatval(substr($registro, 170, 13));
        }
        $nrRetorno = trim(substr($registro, 37, 20));
        if (empty($nrRetorno)){
            self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
            parent::setStatusArquivo(false);
        }
    }

    private function segmentoO($registro){
        $this->retorno->setDtpagamento(substr($registro, 99, 8));
        $vrpago = floatval(substr($registro, 107, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        $nrRetorno = trim(substr($registro, 142, 20));
        if (empty($nrRetorno)){
            self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
            parent::setStatusArquivo(false);
        }
    }

}